@@ -13,7 +13,6 @@
 # language governing permissions and limitations under the License.
 import socket
 
-import botocore.config
 from botocore import args, exceptions
 from botocore.client import ClientEndpointBridge
 from botocore.config import Config
@@ -28,7 +27,8 @@ class TestCreateClientArgs(unittest.TestCase):
         self.event_emitter = mock.Mock(HierarchicalEmitter)
         self.config_store = ConfigValueStore()
         self.args_create = args.ClientArgsCreator(
-            self.event_emitter, None, None, None, None, self.config_store)
+            self.event_emitter, None, None, None, None, self.config_store
+        )
         self.service_name = 'ec2'
         self.region = 'us-west-2'
         self.endpoint_url = 'https://ec2/'
@@ -47,7 +47,7 @@ class TestCreateClientArgs(unittest.TestCase):
         service_model.endpoint_prefix = service_name
         service_model.metadata = {
             'serviceFullName': 'MyService',
-            'protocol': 'query'
+            'protocol': 'query',
         }
         service_model.operation_names = []
         return service_model
@@ -59,7 +59,7 @@ class TestCreateClientArgs(unittest.TestCase):
             'endpoint_url': self.endpoint_url,
             'signing_name': self.service_name,
             'signing_region': self.region,
-            'metadata': {}
+            'metadata': {},
         }
         ret_val.update(**override_kwargs)
         self.bridge.resolve.return_value = ret_val
@@ -74,7 +74,7 @@ class TestCreateClientArgs(unittest.TestCase):
             'credentials': None,
             'scoped_config': {},
             'client_config': None,
-            'endpoint_bridge': self.bridge
+            'endpoint_bridge': self.bridge,
         }
         call_kwargs.update(**override_kwargs)
         return self.args_create.get_client_args(**call_kwargs)
@@ -102,10 +102,11 @@ class TestCreateClientArgs(unittest.TestCase):
 
     def test_compute_s3_config_only_config_store(self):
         self.config_store.set_config_variable(
-            's3', {'use_accelerate_endpoint': True})
+            's3', {'use_accelerate_endpoint': True}
+        )
         self.assertEqual(
             self.args_create.compute_s3_config(None),
-            {'use_accelerate_endpoint': True}
+            {'use_accelerate_endpoint': True},
         )
 
     def test_client_s3_accelerate_from_client_config(self):
@@ -113,30 +114,33 @@ class TestCreateClientArgs(unittest.TestCase):
             self.args_create.compute_s3_config(
                 client_config=Config(s3={'use_accelerate_endpoint': True})
             ),
-            {'use_accelerate_endpoint': True}
+            {'use_accelerate_endpoint': True},
         )
 
     def test_client_s3_accelerate_client_config_overrides_config_store(self):
         self.config_store.set_config_variable(
-            's3', {'use_accelerate_endpoint': False})
+            's3', {'use_accelerate_endpoint': False}
+        )
         self.assertEqual(
             self.args_create.compute_s3_config(
                 client_config=Config(s3={'use_accelerate_endpoint': True})
             ),
             # client_config beats scoped_config
-            {'use_accelerate_endpoint': True}
+            {'use_accelerate_endpoint': True},
         )
 
     def test_max_pool_from_client_config_forwarded_to_endpoint_creator(self):
-        config = botocore.config.Config(max_pool_connections=20)
+        config = Config(max_pool_connections=20)
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(client_config=config)
             self.assert_create_endpoint_call(m, max_pool_connections=20)
 
     def test_proxies_from_client_config_forwarded_to_endpoint_creator(self):
-        proxies = {'http': 'http://foo.bar:1234',
-                   'https': 'https://foo.bar:4321'}
-        config = botocore.config.Config(proxies=proxies)
+        proxies = {
+            'http': 'http://foo.bar:1234',
+            'https': 'https://foo.bar:4321',
+        }
+        config = Config(proxies=proxies)
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(client_config=config)
             self.assert_create_endpoint_call(m, proxies=proxies)
@@ -146,32 +150,43 @@ class TestCreateClientArgs(unittest.TestCase):
         self.service_model.metadata = {'protocol': 'rest-xml'}
         self.bridge.resolve.side_effect = [
             {
-                'region_name': None, 'signature_version': 's3v4',
-                'endpoint_url': 'http://other.com/', 'signing_name': 's3',
-                'signing_region': None, 'metadata': {}
+                'region_name': None,
+                'signature_version': 's3v4',
+                'endpoint_url': 'http://other.com/',
+                'signing_name': 's3',
+                'signing_region': None,
+                'metadata': {},
             },
             {
-                'region_name': 'us-west-2', 'signature_version': 's3v4',
+                'region_name': 'us-west-2',
+                'signature_version': 's3v4',
                 'enpoint_url': 'https://s3-us-west-2.amazonaws.com',
-                'signing_name': 's3', 'signing_region': 'us-west-2',
-                'metadata': {}
-            }
+                'signing_name': 's3',
+                'signing_region': 'us-west-2',
+                'metadata': {},
+            },
         ]
         client_args = self.call_get_client_args(
-            endpoint_url='http://other.com/')
-        self.assertEqual(
-            client_args['client_config'].region_name, 'us-west-2')
+            endpoint_url='http://other.com/'
+        )
+        self.assertEqual(client_args['client_config'].region_name, 'us-west-2')
 
     def test_region_does_not_resolve_if_not_s3_and_endpoint_url_provided(self):
         self.service_model.endpoint_prefix = 'ec2'
         self.service_model.metadata = {'protocol': 'query'}
-        self.bridge.resolve.side_effect = [{
-            'region_name': None, 'signature_version': 'v4',
-            'endpoint_url': 'http://other.com/', 'signing_name': 'ec2',
-            'signing_region': None, 'metadata': {}
-        }]
+        self.bridge.resolve.side_effect = [
+            {
+                'region_name': None,
+                'signature_version': 'v4',
+                'endpoint_url': 'http://other.com/',
+                'signing_name': 'ec2',
+                'signing_region': None,
+                'metadata': {},
+            }
+        ]
         client_args = self.call_get_client_args(
-            endpoint_url='http://other.com/')
+            endpoint_url='http://other.com/'
+        )
         self.assertEqual(client_args['client_config'].region_name, None)
 
     def test_tcp_keepalive_enabled(self):
@@ -179,9 +194,9 @@ class TestCreateClientArgs(unittest.TestCase):
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(scoped_config=scoped_config)
             self.assert_create_endpoint_call(
-                m, socket_options=self.default_socket_options + [
-                    (socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-                ]
+                m,
+                socket_options=self.default_socket_options
+                + [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)],
             )
 
     def test_tcp_keepalive_not_specified(self):
@@ -189,52 +204,46 @@ class TestCreateClientArgs(unittest.TestCase):
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(scoped_config=scoped_config)
             self.assert_create_endpoint_call(
-                m, socket_options=self.default_socket_options)
+                m, socket_options=self.default_socket_options
+            )
 
     def test_tcp_keepalive_explicitly_disabled(self):
         scoped_config = {'tcp_keepalive': 'false'}
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(scoped_config=scoped_config)
             self.assert_create_endpoint_call(
-                m, socket_options=self.default_socket_options)
+                m, socket_options=self.default_socket_options
+            )
 
     def test_tcp_keepalive_enabled_case_insensitive(self):
         scoped_config = {'tcp_keepalive': 'True'}
         with mock.patch('botocore.args.EndpointCreator') as m:
             self.call_get_client_args(scoped_config=scoped_config)
             self.assert_create_endpoint_call(
-                m, socket_options=self.default_socket_options + [
-                    (socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
-                ]
+                m,
+                socket_options=self.default_socket_options
+                + [(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)],
             )
 
     def test_client_config_has_use_dualstack_endpoint_flag(self):
-        self._set_endpoint_bridge_resolve(
-            metadata={
-                'tags': ['dualstack']
-            }
-        )
+        self._set_endpoint_bridge_resolve(metadata={'tags': ['dualstack']})
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('ec2'),
         )
         self.assertTrue(client_args['client_config'].use_dualstack_endpoint)
 
     def test_client_config_has_use_fips_endpoint_flag(self):
-        self._set_endpoint_bridge_resolve(
-            metadata={
-                'tags': ['fips']
-            }
-        )
+        self._set_endpoint_bridge_resolve(metadata={'tags': ['fips']})
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('ec2'),
         )
         self.assertTrue(client_args['client_config'].use_fips_endpoint)
 
-    def test_client_config_has_both_use_fips_and_use_dualstack__endpoint_flags(self):
+    def test_client_config_has_both_use_fips_and_use_dualstack__endpoint_flags(
+        self,
+    ):
         self._set_endpoint_bridge_resolve(
-            metadata={
-                'tags': ['fips', 'dualstack']
-            }
+            metadata={'tags': ['fips', 'dualstack']}
         )
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('ec2'),
@@ -243,125 +252,143 @@ class TestCreateClientArgs(unittest.TestCase):
         self.assertTrue(client_args['client_config'].use_dualstack_endpoint)
 
     def test_s3_override_use_dualstack_endpoint_flag(self):
-        self._set_endpoint_bridge_resolve(
-            metadata={
-                'tags': ['dualstack']
-            }
-        )
+        self._set_endpoint_bridge_resolve(metadata={'tags': ['dualstack']})
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('s3'),
         )
-        self.assertTrue(client_args['client_config'].s3['use_dualstack_endpoint'])
+        self.assertTrue(
+            client_args['client_config'].s3['use_dualstack_endpoint']
+        )
 
     def test_sts_override_resolved_endpoint_for_legacy_region(self):
         self.config_store.set_config_variable(
-            'sts_regional_endpoints', 'legacy')
+            'sts_regional_endpoints', 'legacy'
+        )
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('sts'),
-            region_name='us-west-2', endpoint_url=None
+            region_name='us-west-2',
+            endpoint_url=None,
         )
         self.assertEqual(
-            client_args['endpoint'].host, 'https://sts.amazonaws.com')
+            client_args['endpoint'].host, 'https://sts.amazonaws.com'
+        )
         self.assertEqual(
-            client_args['request_signer'].region_name, 'us-east-1')
+            client_args['request_signer'].region_name, 'us-east-1'
+        )
 
     def test_sts_use_resolved_endpoint_for_nonlegacy_region(self):
         resolved_endpoint = 'https://resolved-endpoint'
         resolved_region = 'resolved-region'
         self._set_endpoint_bridge_resolve(
-            endpoint_url=resolved_endpoint,
-            signing_region=resolved_region
+            endpoint_url=resolved_endpoint, signing_region=resolved_region
         )
         self.config_store.set_config_variable(
-            'sts_regional_endpoints', 'legacy')
+            'sts_regional_endpoints', 'legacy'
+        )
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('sts'),
-            region_name='ap-east-1', endpoint_url=None
+            region_name='ap-east-1',
+            endpoint_url=None,
         )
         self.assertEqual(client_args['endpoint'].host, resolved_endpoint)
         self.assertEqual(
-            client_args['request_signer'].region_name, resolved_region)
+            client_args['request_signer'].region_name, resolved_region
+        )
 
     def test_sts_use_resolved_endpoint_for_regional_configuration(self):
         resolved_endpoint = 'https://resolved-endpoint'
         resolved_region = 'resolved-region'
         self._set_endpoint_bridge_resolve(
-            endpoint_url=resolved_endpoint,
-            signing_region=resolved_region
+            endpoint_url=resolved_endpoint, signing_region=resolved_region
         )
         self.config_store.set_config_variable(
-            'sts_regional_endpoints', 'regional')
+            'sts_regional_endpoints', 'regional'
+        )
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('sts'),
-            region_name='us-west-2', endpoint_url=None
+            region_name='us-west-2',
+            endpoint_url=None,
         )
         self.assertEqual(client_args['endpoint'].host, resolved_endpoint)
         self.assertEqual(
-            client_args['request_signer'].region_name, resolved_region)
+            client_args['request_signer'].region_name, resolved_region
+        )
 
     def test_sts_with_endpoint_override_and_legacy_configured(self):
         override_endpoint = 'https://override-endpoint'
         self._set_endpoint_bridge_resolve(endpoint_url=override_endpoint)
         self.config_store.set_config_variable(
-            'sts_regional_endpoints', 'legacy')
+            'sts_regional_endpoints', 'legacy'
+        )
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('sts'),
-            region_name='us-west-2', endpoint_url=override_endpoint
+            region_name='us-west-2',
+            endpoint_url=override_endpoint,
         )
         self.assertEqual(client_args['endpoint'].host, override_endpoint)
 
     def test_sts_http_scheme_for_override_endpoint(self):
         self.config_store.set_config_variable(
-            'sts_regional_endpoints', 'legacy')
+            'sts_regional_endpoints', 'legacy'
+        )
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('sts'),
-            region_name='us-west-2', endpoint_url=None, is_secure=False,
-
+            region_name='us-west-2',
+            endpoint_url=None,
+            is_secure=False,
         )
         self.assertEqual(
-            client_args['endpoint'].host, 'http://sts.amazonaws.com')
+            client_args['endpoint'].host, 'http://sts.amazonaws.com'
+        )
 
     def test_sts_regional_endpoints_defaults_to_legacy_if_not_set(self):
-        self.config_store.set_config_variable(
-            'sts_regional_endpoints', None)
+        self.config_store.set_config_variable('sts_regional_endpoints', None)
         client_args = self.call_get_client_args(
             service_model=self._get_service_model('sts'),
-            region_name='us-west-2', endpoint_url=None
+            region_name='us-west-2',
+            endpoint_url=None,
         )
         self.assertEqual(
-            client_args['endpoint'].host, 'https://sts.amazonaws.com')
+            client_args['endpoint'].host, 'https://sts.amazonaws.com'
+        )
         self.assertEqual(
-            client_args['request_signer'].region_name, 'us-east-1')
+            client_args['request_signer'].region_name, 'us-east-1'
+        )
 
     def test_invalid_sts_regional_endpoints(self):
         self.config_store.set_config_variable(
-            'sts_regional_endpoints', 'invalid')
+            'sts_regional_endpoints', 'invalid'
+        )
         with self.assertRaises(
-                exceptions.InvalidSTSRegionalEndpointsConfigError):
+            exceptions.InvalidSTSRegionalEndpointsConfigError
+        ):
             self.call_get_client_args(
                 service_model=self._get_service_model('sts'),
-                region_name='us-west-2', endpoint_url=None
+                region_name='us-west-2',
+                endpoint_url=None,
             )
 
     def test_provides_total_max_attempts(self):
-        config = botocore.config.Config(retries={'total_max_attempts': 10})
+        config = Config(retries={'total_max_attempts': 10})
         client_args = self.call_get_client_args(client_config=config)
         self.assertEqual(
-            client_args['client_config'].retries['total_max_attempts'], 10)
+            client_args['client_config'].retries['total_max_attempts'], 10
+        )
 
     def test_provides_total_max_attempts_has_precedence(self):
-        config = botocore.config.Config(retries={'total_max_attempts': 10,
-                                                 'max_attempts': 5})
+        config = Config(retries={'total_max_attempts': 10, 'max_attempts': 5})
         client_args = self.call_get_client_args(client_config=config)
         self.assertEqual(
-            client_args['client_config'].retries['total_max_attempts'], 10)
+            client_args['client_config'].retries['total_max_attempts'], 10
+        )
         self.assertNotIn('max_attempts', client_args['client_config'].retries)
 
     def test_provide_retry_config_maps_total_max_attempts(self):
-        config = botocore.config.Config(retries={'max_attempts': 10})
+        config = Config(retries={'max_attempts': 10})
         client_args = self.call_get_client_args(client_config=config)
         self.assertEqual(
-            client_args['client_config'].retries['total_max_attempts'], 11)
+            client_args['client_config'].retries['total_max_attempts'], 11
+        )
         self.assertNotIn('max_attempts', client_args['client_config'].retries)
 
     def test_can_merge_max_attempts(self):
@@ -390,9 +417,9 @@ class TestCreateClientArgs(unittest.TestCase):
         self.assertEqual(config.retries['total_max_attempts'], 2)
 
     def test_max_attempts_unset_if_retries_is_none(self):
-        config = self.call_get_client_args(
-            client_config=Config(retries=None)
-        )['client_config']
+        config = self.call_get_client_args(client_config=Config(retries=None))[
+            'client_config'
+        ]
         self.assertEqual(config.retries, {'mode': 'legacy'})
 
     def test_retry_mode_set_on_config_store(self):
@@ -429,3 +456,222 @@ class TestCreateClientArgs(unittest.TestCase):
             client_config=Config(retries={'mode': 'standard'})
         )['client_config']
         self.assertEqual(config.retries['mode'], 'standard')
+
+
+class TestEndpointResolverBuiltins(unittest.TestCase):
+    def setUp(self):
+        event_emitter = mock.Mock(HierarchicalEmitter)
+        self.config_store = ConfigValueStore()
+        self.args_create = args.ClientArgsCreator(
+            event_emitter=event_emitter,
+            user_agent=None,
+            response_parser_factory=None,
+            loader=None,
+            exceptions_factory=None,
+            config_store=self.config_store,
+        )
+        self.bridge = ClientEndpointBridge(
+            endpoint_resolver=mock.Mock(),
+            scoped_config=None,
+            client_config=Config(),
+            default_endpoint=None,
+            service_signing_name=None,
+            config_store=self.config_store,
+        )
+        # assume a legacy endpoint resolver that uses the builtin
+        # endpoints.json file
+        self.bridge.endpoint_resolver.uses_builtin_data = True
+
+    def call_compute_endpoint_resolver_builtin_defaults(self, **overrides):
+        defaults = {
+            'region_name': 'ca-central-1',
+            'service_name': 'fooservice',
+            's3_config': {},
+            'endpoint_bridge': self.bridge,
+            'client_endpoint_url': None,
+            'legacy_endpoint_url': 'https://my.legacy.endpoint.com',
+        }
+        kwargs = {**defaults, **overrides}
+        return self.args_create.compute_endpoint_resolver_builtin_defaults(
+            **kwargs
+        )
+
+    def test_builtins_defaults(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults()
+
+        self.assertEqual(bins['AWS::Region'], 'ca-central-1')
+        self.assertEqual(bins['AWS::UseFIPS'], False)
+        self.assertEqual(bins['AWS::UseDualStack'], False)
+        self.assertEqual(bins['AWS::STS::UseGlobalEndpoint'], True)
+        self.assertEqual(bins['AWS::S3::UseGlobalEndpoint'], False)
+        self.assertEqual(bins['AWS::S3::Accelerate'], False)
+        self.assertEqual(bins['AWS::S3::ForcePathStyle'], False)
+        self.assertEqual(bins['AWS::S3::UseArnRegion'], True)
+        self.assertEqual(bins['AWS::S3Control::UseArnRegion'], False)
+        self.assertEqual(
+            bins['AWS::S3::DisableMultiRegionAccessPoints'], False
+        )
+        self.assertEqual(bins['SDK::Endpoint'], None)
+
+    def test_aws_region(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            region_name='my-region-1',
+        )
+        self.assertEqual(bins['AWS::Region'], 'my-region-1')
+
+    def test_aws_use_fips_when_config_is_set_true(self):
+        self.config_store.set_config_variable('use_fips_endpoint', True)
+        bins = self.call_compute_endpoint_resolver_builtin_defaults()
+        self.assertEqual(bins['AWS::UseFIPS'], True)
+
+    def test_aws_use_fips_when_config_is_set_false(self):
+        self.config_store.set_config_variable('use_fips_endpoint', False)
+        bins = self.call_compute_endpoint_resolver_builtin_defaults()
+        self.assertEqual(bins['AWS::UseFIPS'], False)
+
+    def test_aws_use_dualstack_when_config_is_set_true(self):
+        self.bridge.client_config = Config(s3={'use_dualstack_endpoint': True})
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            service_name='s3-control'
+        )
+        self.assertEqual(bins['AWS::UseDualStack'], True)
+
+    def test_aws_use_dualstack_when_config_is_set_false(self):
+        self.bridge.client_config = Config(
+            s3={'use_dualstack_endpoint': False}
+        )
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            service_name='s3-control'
+        )
+        self.assertEqual(bins['AWS::UseDualStack'], False)
+
+    def test_aws_use_dualstack_when_non_dualstack_service(self):
+        self.bridge.client_config = Config(s3={'use_dualstack_endpoint': True})
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            service_name='other-service'
+        )
+        self.assertEqual(bins['AWS::UseDualStack'], False)
+
+    def test_aws_sts_global_endpoint_with_default_and_legacy_region(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            region_name='us-west-2',
+        )
+        self.assertEqual(bins['AWS::STS::UseGlobalEndpoint'], True)
+
+    def test_aws_sts_global_endpoint_with_default_and_nonlegacy_region(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            region_name='eu-south-1',
+        )
+        self.assertEqual(bins['AWS::STS::UseGlobalEndpoint'], False)
+
+    def test_aws_sts_global_endpoint_with_nondefault_config(self):
+        self.config_store.set_config_variable(
+            'sts_regional_endpoints', 'regional'
+        )
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            region_name='us-west-2',
+        )
+        self.assertEqual(bins['AWS::STS::UseGlobalEndpoint'], False)
+
+    def test_s3_global_endpoint(self):
+        # The only reason for this builtin to not have the default value
+        # (False) is that the ``_should_force_s3_global`` method
+        # returns True.
+        self.args_create._should_force_s3_global = mock.Mock(return_value=True)
+        bins = self.call_compute_endpoint_resolver_builtin_defaults()
+        self.assertTrue(bins['AWS::S3::UseGlobalEndpoint'])
+        self.args_create._should_force_s3_global.assert_called_once()
+
+    def test_s3_accelerate_with_config_set_true(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'use_accelerate_endpoint': True},
+        )
+        self.assertEqual(bins['AWS::S3::Accelerate'], True)
+
+    def test_s3_accelerate_with_config_set_false(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'use_accelerate_endpoint': False},
+        )
+        self.assertEqual(bins['AWS::S3::Accelerate'], False)
+
+    def test_force_path_style_with_config_set_to_path(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'addressing_style': 'path'},
+        )
+        self.assertEqual(bins['AWS::S3::ForcePathStyle'], True)
+
+    def test_force_path_style_with_config_set_to_auto(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'addressing_style': 'auto'},
+        )
+        self.assertEqual(bins['AWS::S3::ForcePathStyle'], False)
+
+    def test_force_path_style_with_config_set_to_virtual(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'addressing_style': 'virtual'},
+        )
+        self.assertEqual(bins['AWS::S3::ForcePathStyle'], False)
+
+    def test_use_arn_region_with_config_set_false(self):
+        # These two builtins both take their value from the ``use_arn_region``
+        # in the S3 configuration, but have different default values.
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'use_arn_region': False},
+        )
+        self.assertEqual(bins['AWS::S3::UseArnRegion'], False)
+        self.assertEqual(bins['AWS::S3Control::UseArnRegion'], False)
+
+    def test_use_arn_region_with_config_set_true(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'use_arn_region': True},
+        )
+        self.assertEqual(bins['AWS::S3::UseArnRegion'], True)
+        self.assertEqual(bins['AWS::S3Control::UseArnRegion'], True)
+
+    def test_disable_mrap_with_config_set_true(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'s3_disable_multiregion_access_points': True},
+        )
+        self.assertEqual(bins['AWS::S3::DisableMultiRegionAccessPoints'], True)
+
+    def test_disable_mrap_with_config_set_false(self):
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            s3_config={'s3_disable_multiregion_access_points': False},
+        )
+        self.assertEqual(
+            bins['AWS::S3::DisableMultiRegionAccessPoints'], False
+        )
+
+    def test_sdk_endpoint_both_inputs_set(self):
+        # assume a legacy endpoint resolver that uses a customized
+        # endpoints.json file
+        self.bridge.endpoint_resolver.uses_builtin_data = False
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            client_endpoint_url='https://my.client.endpoint.com',
+            legacy_endpoint_url='https://my.legacy.endpoint.com',
+        )
+        self.assertEqual(
+            bins['SDK::Endpoint'], 'https://my.client.endpoint.com'
+        )
+
+    def test_sdk_endpoint_legacy_set_with_builtin_data(self):
+        # assume a legacy endpoint resolver that uses a customized
+        # endpoints.json file
+        self.bridge.endpoint_resolver.uses_builtin_data = False
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            client_endpoint_url=None,
+            legacy_endpoint_url='https://my.legacy.endpoint.com',
+        )
+        self.assertEqual(
+            bins['SDK::Endpoint'], 'https://my.legacy.endpoint.com'
+        )
+
+    def test_sdk_endpoint_legacy_set_without_builtin_data(self):
+        # assume a legacy endpoint resolver that uses the builtin
+        # endpoints.json file
+        self.bridge.endpoint_resolver.uses_builtin_data = True
+        bins = self.call_compute_endpoint_resolver_builtin_defaults(
+            client_endpoint_url=None,
+            legacy_endpoint_url='https://my.legacy.endpoint.com',
+        )
+        self.assertEqual(bins['SDK::Endpoint'], None)
